@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 """
 SetupHelper -- helper module to automate setup boilerplate
-Copyright (C) 2008 by Peter A. Donis
+Copyright (C) 2008-2009 by Peter A. Donis
 
 Released under the Python Software Foundation License.
 
@@ -175,14 +175,27 @@ __non_packages__       -- list of subdirectories of the setup directory
 
 __ext_modules__        -- list of extension module objects; this should
                           not be needed, use __ext_names__
-__ext_names__          -- list of path names to extension modules
-                          (starting from the setup.py directory); the
-                          source code for each module is found by using
-                          the __ext_src__ variable
-__ext_src__            -- pathname to extension source files, usually
-                          relative (meaning relative to the extension
-                          directory); defaults to 'src', meaning the
-                          'src' subdirectory of the extension directory
+__ext_names__          -- list of names of extension modules; dotted names
+                          are interpreted as modules within packages (so
+                          ``foo`` is a top-level module, but ``pkg.foo`` is
+                          a module in package ``pkg``); the source code for
+                          each module is found by using the __ext_src__
+                          variable to name a subdirectory of the extension
+                          module directory (which will be the corresponding
+                          source root or package directory).
+__ext_src__            -- pathname to extension source files, relative to
+                          the source root; if omitted, extension source files
+                          will be looked for in the 'src' subdirectory of each
+                          extension module directory (this should be the usual
+                          case, although it is somewhat different that the
+                          assumed default in the Python documentation; it
+                          seems neater to keep each extension module's source
+                          code near the module's own location in the build
+                          tree, and difficulties that might otherwise arise
+                          with doing this are avoided by SetupHelper's
+                          automatically excluding extension source directories
+                          from the list of packages, as noted under the
+                          ``__package_data__`` variable above).
 
 __scripts__            -- list of scripts to be included; this should
                           generally not be used, use __script_root__
@@ -250,7 +263,7 @@ import sys
 import os
 from itertools import imap, izip
 
-SETUPHELPER_VERSION = (0, 4, 3)
+SETUPHELPER_VERSION = (0, 5)
 
 def version_tuple(ver_s, dev=False):
     """
@@ -450,13 +463,33 @@ archive_extensions = {
     'linux2': [".tar.gz", ".tar.bz2", ".zip", ".tar"],
     'win32': [".exe", ".zip", ".tar.gz", ".tar"] }
 
+install_scripts_dir = None # see ugly hack below
+
 def distutils_default():
     """ Support function for Python-provided distutils. """
     
+    import distutils.command.install_scripts
+    from distutils.command.install_scripts import install_scripts as _install_scripts
     from distutils.core import setup, Extension
     import urllib2
     import re
     
+    class install_scripts(_install_scripts):
+        """
+        Captures the script installation directory so that post-install scripts
+        will be run correctly. Yes, this is an ugly hack; if distutils would
+        expose these directories to setup scripts all this would be unnecessary.
+        """
+        
+        def finalize_options(self):
+            global install_scripts_dir
+            _install_scripts.finalize_options(self)
+            install_scripts_dir = self.install_dir
+            print "Installing scripts to %s ..." % install_scripts_dir
+    
+    # Monkeypatch distutils with our hacked class above
+    distutils.command.install_scripts.install_scripts = install_scripts
+    
     class RequiresHelper(object):
         """
         Helper class to handle requires keyword. Should be
@@ -884,9 +917,34 @@ class SetupHelper(object):
         'requires_class',
         'setup_func' ]
     
+    dirsep = '/'
+    pkgsep = '.'
+    
+    verbose = False
+    
+    def dir_to_os(self, dirname):
+        """ Converts distutils dirname to os dirname. """
+        return os.path.join(*dirname.split(self.dirsep))
+    
     def dir_to_pkg(self, dirname):
         """ Converts relative directory path into dotted package name. """
-        return dirname.replace('/', '.')
+        return dirname.replace(self.dirsep, self.pkgsep)
+    
+    def os_to_dir(self, dirname):
+        """ Converts os directory path into distutils dirname. """
+        return dirname.replace(os.sep, self.dirsep)
+    
+    def os_to_pkg(self, dirname):
+        """ Converts os directory path into dotted package name. """
+        return dirname.replace(os.sep, self.pkgsep)
+    
+    def pkg_to_dir(self, pkgname):
+        """ Converts dotted package name into relative directory path. """
+        return pkgname.replace(self.pkgsep, self.dirsep)
+    
+    def pkg_to_os(self, pkgname):
+        """ Converts dotted package name to os dirname. """
+        return os.path.join(*pkgname.split(self.pkgsep))
     
     def search_classifiers(self, s):
         """
@@ -1074,26 +1132,27 @@ class SetupHelper(object):
         
         # Figure out the extension modules list (do this first so we can filter
         # out extension directories from the package data list below)
-        self.__ext_modules__ = []
+        if not hasattr(self, '__ext_modules__'):
+            self.__ext_modules__ = []
         self.__ext_sourcedirs__ = []
         if hasattr(self, '__ext_names__'):
-            if not hasattr(self, '__ext_src__'):
-                self.__ext_src__ = 'src'
-            if os.path.isabs(self.__ext_src__):
-                ext_src_func = lambda extdir: self.__ext_src__
+            if hasattr(self, '__ext_src__'):
+                ext_src_func = lambda extname: self.__ext_src__
             else:
-                ext_src_func = lambda extdir: os.path.normpath(
-                    os.path.join(extdir, self.__ext_src__))
-            for extpath in self.__ext_names__:
-                extdir, extname = os.path.split(extpath)
-                srcdir = ext_src_func(extdir)
+                ext_src_func = lambda extname: os.path.normpath(
+                    os.path.join(os.path.split(self.pkg_to_os(extname))[0], 'src'))
+            for extname in self.__ext_names__:
+                srcdir = ext_src_func(extname)
                 if os.path.isdir(srcdir):
-                    self.__ext_modules__.append(self.__ext_class__(extpath,
+                    self.__ext_modules__.append(self.__ext_class__(extname,
                         sources=[os.path.join(srcdir, filename)
                             for filename in os.listdir(srcdir) if filename.endswith(".c")]))
                     self.__ext_sourcedirs__.append(srcdir)
-            del self.__ext_src__
+            if hasattr(self, '__ext_src__'):
+                del self.__ext_src__
             del self.__ext_names__
+        # TODO: if __ext_modules__ exists but no __ext_names__, parse it to
+        # populate __ext_sourcedirs__
         if not self.__ext_modules__:
             del self.__ext_modules__
         del self.__ext_class__
@@ -1125,11 +1184,11 @@ class SetupHelper(object):
             for pkgdir in self.__package_dirs__:
                 for directory, subdirs, files in os.walk(pkgdir):
                     if process_packages and ("__init__.py" in files):
-                        self.__packages__.append(self.dir_to_pkg(directory))
+                        self.__packages__.append(self.os_to_pkg(directory))
                     elif process_packagedata and ("." not in directory) \
                         and (directory not in self.__ext_sourcedirs__):
                             parentdir, thisdir = os.path.split(directory)
-                            pkg = self.dir_to_pkg(parentdir)
+                            pkg = self.os_to_pkg(parentdir)
                             if not pkg in self.__package_data__:
                                 self.__package_data__[pkg] = []
                             self.__package_data__[pkg].append("%s/*.*" % thisdir)
@@ -1154,7 +1213,7 @@ class SetupHelper(object):
                     if "." not in directory:
                         for filename in files:
                             if not filename.startswith("."):
-                                self.__scripts__.append("%s/%s" % (directory, filename))
+                                self.__scripts__.append("%s/%s" % (self.os_to_dir(directory), filename))
         if hasattr(self, '__script_root__'):
             del self.__script_root__
         if not hasattr(self, '__data_files__'):
@@ -1301,6 +1360,9 @@ class SetupHelper(object):
         function call.
         """
         
+        if self.verbose:
+            for key, value in self.__setup_args__.iteritems():
+                print key, ':', value
         self.__setup_func__(**self.__setup_args__)
     
     def do_post_install(self):
@@ -1311,14 +1373,19 @@ class SetupHelper(object):
         """
         
         if hasattr(self, '__post_install__') and self.__post_install__:
-            print "Running post-install scripts..."
+            print "Running post-install scripts in %s ..." % install_scripts_dir
             for scriptname in self.__post_install__:
-                os.system(scriptname)
+                # Use ugly hack above to get script dir from distutils to be sure we're grabbing the right script
+                os.system(os.path.join(install_scripts_dir, scriptname))
 
 # TODO: convert long option of the form --<option-name>=<value> to keyword argument option_name=value
 
 def setup_main(vars, **kwargs):
     helper = SetupHelper(vars, **kwargs)
+    for opt in ('-v', '--verbose'):
+        if opt in sys.argv:
+            sys.argv.remove(opt)
+            helper.verbose = True
     helper.do_setup()
     if 'install' in sys.argv:
         helper.do_post_install()
